from typing import List

from workspaces.capability_interfaces import (
    CapabilityInfoIF,
    CapabilityExecutionIF,
    CapabilityIF,
    CapabilityRequestIF,
    CapabilityName,
)
from workspaces.helpers import ExecutionState
from workspaces.product_interfaces import FutureProductIF
from workspaces.schema import Capability, CapabilityExecution
from workspaces.services import CapabilityService


class FakeCapabilityInfo(CapabilityInfoIF):
    capabilities = [
        Capability(
            id=1,
            name="null",
            steps="prepare and run workflow null\nawait workflow\n",
            max_jobs=2,
        )
    ]

    def create_capability(
        self, name: CapabilityName, steps: List["CapabilityStepType"], max_jobs: int
    ) -> CapabilityIF:
        capability = Capability(len(self.capabilities), name, steps, max_jobs)
        self.capabilities.append(capability)
        return capability

    def create_capability_request(
        self,
        capability_name: str,
        parameters: List["ParameterIF"] = None,
        future_products: List[FutureProductIF] = None,
    ) -> CapabilityRequestIF:
        capability = self.lookup_capability(capability_name)
        request = capability.create_request(parameters, future_products)
        return request

    def create_execution(self, request: CapabilityRequestIF) -> CapabilityExecutionIF:
        most_recent_version = request.versions[-1]

        execution = CapabilityExecution(
            id=len(most_recent_version.executions) + 1,
            state=ExecutionState.Ready.name,
            version=most_recent_version,
            current_step=0,
            # ensure that we have a copy of the step sequence as it was when the execution started
            steps=request.capability.steps,
        )

        most_recent_version.executions.append(execution)

        return execution

    def lookup_capability(self, capability_name: str) -> CapabilityIF:
        return next(c for c in self.capabilities if c.name == capability_name)

    def lookup_execution(self, execution_id: int) -> CapabilityExecutionIF:
        for capability in self.capabilities:
            for version in capability.versions:
                for execution in version.executions:
                    if execution_id == execution.id:
                        return execution


info = FakeCapabilityInfo()


def test_capability_request_creation():
    cs = CapabilityService(info)
    req = cs.create_request("null")
    assert req in info.capabilities[0].requests
